'''Command line interface'''

import argparse
import logging
from typing import List, Union
from px_device_identity.device.config import CONFIG_DIR, KEY_DIR

from .config import PORT, HOST

log = logging.getLogger(__name__)


def get_cl_arguments(args: Union[List, None] = None):
    '''
    Build command line interface and parse input
    '''
    parser = argparse.ArgumentParser()
    parser.add_argument("-p", "--port", type=int, default=PORT)
    parser.add_argument("-ho", "--host", type=str, default=HOST)
    parser.add_argument("-kd", "--key-dir", type=str, default=KEY_DIR)
    parser.add_argument("-cd", "--config-dir", type=str, default=CONFIG_DIR)
    parser.add_argument("-d", "--debug", type=bool, default=False,
                        help="Turn on debug messages"
                        )
    
    args = parser.parse_args()

    return {
        "port": args.port,
        "host": args.host,
        "key_dir": args.key_dir,
        "config_dir": args.config_dir,
        "debug": args.debug,
    }


'''PantherX Device Identity Service'''

import logging
import platform
import sys

import pkg_resources

from flask import Flask, request, jsonify, make_response, g
from px_device_identity import Device, is_superuser_or_quit, DeviceProperties
from px_device_identity.errors import SigningError
from werkzeug.middleware.proxy_fix import ProxyFix
from waitress import serve

from .cli import get_cl_arguments
from .log import *

log = logging.getLogger(__name__)

opsys = platform.system()
version = pkg_resources.require("px-device-identity-service")[0].version

app = Flask(__name__)
app.wsgi_app = ProxyFix(app.wsgi_app)

@app.before_request
def before_request():
    g.key_dir = app.config['key_dir']
    g.config_dir = app.config['config_dir']
    g.debug = app.config['debug']

@app.after_request
def cors_headers(response):
    response.headers['Access-Control-Allow-Origin'] = '127.0.0.1'
    if request.method == 'OPTIONS':
        response.headers['Access-Control-Allow-Methods'] = response.headers.get('Allow', '127.0.0.1')
        response.headers['Access-Control-Allow-Headers'] = request.headers.get('Access-Control-Request-Headers', '127.0.0.1')
        response.headers['Access-Control-Max-Age'] = '86400'  # 24 hours
    return response

@app.route('/sign', methods=['POST', 'OPTIONS'])
def sign():
    if request.method == 'OPTIONS':
        return make_response('', 204)

    log.info("Received new signing request.")
    log.info(request.json)

    provided_device_id = request.json.get("deviceId", None)
    message = request.json.get("message", None)

    if provided_device_id is None:
        return make_response(jsonify(description='Could not find required deviceId.'), 400)

    if message is None:
        return make_response(jsonify(description='Could not find required message.'), 400)

    try:
        device = Device(
            key_dir=g.key_dir,
            config_dir=g.config_dir,
        )
        device_properties: 'DeviceProperties' = device.properties
        actual_device_id = device_properties.id

        log.debug(
            f"Determined current key ({device_properties.key_type}) security to be {device_properties.key_security}"
        )

        if actual_device_id and actual_device_id == provided_device_id:
            log.info(f"=> Start signing process for {actual_device_id}")
            try:
                signed = device.sign(message)
                if signed:
                    return make_response(jsonify(signature=str(signed)), 202)
                else:
                    return make_response(jsonify(error='Unable to sign authentication request.'), 503)
            except SigningError as err:
                log.error(err)
                return make_response(jsonify(title='Unable to sign authentication request.', message=err.message), 503)

        else:
            log.error(f"Device ID {actual_device_id} does not match with provided ID {provided_device_id}.")
            return make_response(jsonify(description='Device ID does not match with provided ID.'), 400)
    except:
        log.error(sys.exc_info()[0])
        return make_response(jsonify(description='Error occurred during the signing process.'), 404)

def main():
    '''Runs PantherX Device Identity Service'''
    log.info('------')
    log.info('Welcome to PantherX Device Identity Service')
    log.info(f"v{version}")
    log.info('------')

    is_superuser_or_quit()
    
    cl_arguments = get_cl_arguments()
    port = cl_arguments['port']
    host = cl_arguments['host']
    key_dir = cl_arguments['key_dir']
    config_dir = cl_arguments['config_dir']
    debug = cl_arguments['debug']
    
    app.config['key_dir'] = key_dir
    app.config['config_dir'] = config_dir
    app.config['debug'] = debug
    
    if debug:
        ch.setLevel(logging.DEBUG)

    serve(app, host=host, port=port)

if __name__ == '__main__':
    main()



import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

VERSION = '0.11.6'
PACKAGE_NAME = 'px-device-identity-service'
AUTHOR = 'Franz Geffke'
AUTHOR_EMAIL = 'franz@pantherx.org'
URL = 'https://git.pantherx.org/development/applications/px-device-identity-service'

LICENSE = 'MIT'
DESCRIPTION = 'Provides access to device signing capabilities via REST API.'
LONG_DESCRIPTION = (HERE / "README.md").read_text()
LONG_DESC_TYPE = "text/markdown"

INSTALL_REQUIRES = [
    'flask',
    'pyyaml',
    'waitress',
    'werkzeug',
    "px-device-identity @ https://source-git-pantherx-org.s3.eu-central-1.amazonaws.com/px-device-identity_latest.tgz",
]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    license=LICENSE,
    author_email=AUTHOR_EMAIL,
    url=URL,
    install_requires=INSTALL_REQUIRES,
    entry_points={
        'console_scripts': ['px-device-identity-service=px_device_identity_service.main:main'],
    },
    packages=find_packages(),
    zip_safe=False
)



import logging
from logging.handlers import SysLogHandler, RotatingFileHandler
from platform import system

opsys = system()


log = logging.getLogger('px_device_identity_service')
formatter = logging.Formatter(
    "%(asctime)s - %(name)s - %(levelname)s: %(message)s", "%Y-%m-%d %H:%M:%S"
)
formatter_cli = logging.Formatter('%(levelname)s: %(message)s')

if opsys == 'Linux':
    import syslog

    # On Linux we log all events to file
    fh = RotatingFileHandler('/var/log/px-device-identity-service.log', maxBytes=10000, backupCount=1)
    fh.setLevel(logging.INFO)
    fh.setFormatter(formatter)
    log.addHandler(fh)

    #On Linux we engage syslog
    sh = SysLogHandler()
    sh.setLevel(logging.WARNING)
    sh.setFormatter(formatter)
    log.addHandler(sh)

ch = logging.StreamHandler()
ch.setLevel(logging.INFO)
ch.setFormatter(formatter_cli)
log.addHandler(ch)
